@@ -15,15 +15,15 @@ You should have received a copy of the GNU Lesser General Public License along
 with this program.  If not, see <https://www.gnu.org/licenses/>.
 """
 
+import numpy as np
+import pandas as pd
 from .scope import StaticScope
 from .vect import highv
 from collections import deque
 from dataclasses import dataclass, field
 from numba import njit
 from numpy.typing import NDArray
-from typing import Callable, NamedTuple, Optional, Union
-import numpy as np
-import pandas as pd
+from typing import Callable, NamedTuple, Optional
 
 
 @njit
@@ -212,7 +212,7 @@ def log_profit_factor(changes: NDArray[np.float_]) -> np.floating:
 
 
 @njit
-def sharpe_ratio(changes: NDArray[np.float_]) -> Union[float, np.floating]:
+def sharpe_ratio(changes: NDArray[np.float_]) -> np.floating:
     """Computes the
     `Sharpe Ratio <https://en.wikipedia.org/wiki/Sharpe_ratio>`_.
 
@@ -220,10 +220,10 @@ def sharpe_ratio(changes: NDArray[np.float_]) -> Union[float, np.floating]:
         changes: Array of differences between each bar and the previous bar.
     """
     if not len(changes):
-        return 0.0
+        return np.float32(0)
     std = np.std(changes)
     if std == 0:
-        return 0.0
+        return np.float32(0)
     return np.mean(changes) / std
 
 
@@ -647,8 +647,10 @@ class EvalMetrics:
             <https://en.wikipedia.org/wiki/Ulcer_index>`_, computed per bar.
         upi: `Ulcer Performance Index
             <https://en.wikipedia.org/wiki/Ulcer_index>`_, computed per bar.
-        equity_r2: R^2 of the equity curve, computed per bar.
-        std_error: Standard error, computed per bar.
+        equity_r2: R^2 of the equity curve, computed per bar on market values
+            of portfolio.
+        std_error: Standard error, computed per bar on market values of
+            portfolio.
     """
 
     trade_count: int = field(default=0)
