@@ -15,6 +15,9 @@ You should have received a copy of the GNU Lesser General Public License along
 with this program.  If not, see <https://www.gnu.org/licenses/>.
 """
 
+import dataclasses
+import numpy as np
+import pandas as pd
 from .cache import CacheDateFields
 from .common import (
     BarData,
@@ -29,6 +32,7 @@ from .common import (
     to_decimal,
     to_seconds,
     verify_data_source_columns,
+    verify_date_range,
 )
 from .config import StrategyConfig
 from .context import (
@@ -65,9 +69,6 @@ from typing import (
     Sequence,
     Union,
 )
-import dataclasses
-import numpy as np
-import pandas as pd
 
 
 def _decorate_execution_fn(fn: Callable[[ExecContext], None]) -> Callable:
@@ -212,10 +213,11 @@ class BacktestMixin:
             for exec in executions
             if exec.fn is not None and sym in exec.symbols
         )
-        test_data = test_data.reset_index(drop=True).set_index(
-            [DataCol.SYMBOL.value, DataCol.DATE.value]
+        test_data = (
+            test_data.reset_index(drop=True)
+            .set_index([DataCol.SYMBOL.value, DataCol.DATE.value])
+            .sort_index()
         )
-        test_data = test_data.sort_index()
         test_symbols = frozenset(
             exec_symbol.symbol for exec_symbol in exec_symbols
         )
@@ -598,17 +600,7 @@ class BacktestMixin:
         if price_type == float or price_type == int or price_type == Decimal:
             return to_decimal(price)  # type: ignore[arg-type]
         if callable(price):
-            scope = StaticScope.instance()
-            default_col_data = col_scope.fetch_dict(
-                symbol, scope.default_data_cols, end_index
-            )
-            custom_col_data = col_scope.fetch_dict(
-                symbol, scope.custom_data_cols, end_index
-            )
-            bar_data = BarData(
-                **default_col_data,  # type: ignore[arg-type]
-                **custom_col_data,  # type: ignore[arg-type]
-            )
+            bar_data = col_scope.bar_data_from_data_columns(symbol, end_index)
             return to_decimal(price(bar_data))
         raise ValueError(f"Unknown price: {price_type}")
 
@@ -834,7 +826,7 @@ class Strategy(
         self._data_source = data_source
         self._start_date = to_datetime(start_date)
         self._end_date = to_datetime(end_date)
-        self._verify_date_range(self._start_date, self._end_date)
+        verify_date_range(self._start_date, self._end_date)
         if config is not None:
             self._verify_config(config)
             self._config = config
@@ -869,19 +861,11 @@ class Strategy(
         if config.bootstrap_sample_size <= 0:
             raise ValueError("bootstrap_sample_size must be greater than 0.")
 
-    def _verify_date_range(self, start_date: datetime, end_date: datetime):
-        if start_date > end_date:
-            raise ValueError(
-                f"start_date ({start_date}) must be before end_date "
-                f"({end_date})."
-            )
-
     def _verify_data_source(
         self, data_source: Union[DataSource, pd.DataFrame]
     ):
         if isinstance(data_source, pd.DataFrame):
             verify_data_source_columns(data_source)
-            return
         elif not isinstance(data_source, DataSource):
             raise TypeError(f"Invalid data_source type: {type(data_source)}")
 
@@ -1027,8 +1011,9 @@ class Strategy(
                 - ``"w"``/``"week"``: weeks
 
                 An example timeframe string is ``1h 30m``.
-            between_time: ``tuple[str, str]`` of times of day
-                (e.g. 9:00-9:30) used to filter the backtesting data.
+            between_time: ``tuple[str, str]`` of times of day e.g.
+                ('9:30', '16:00') used to filter the backtesting data
+                (inclusive).
             days: Days (e.g. ``"mon"``, ``"tues"`` etc.) used to filter the
                 backtesting data.
             lookahead: Number of bars in the future of the target prediction.
@@ -1110,8 +1095,9 @@ class Strategy(
                 - ``"w"``/``"week"``: weeks
 
                 An example timeframe string is ``1h 30m``.
-            between_time: ``tuple[str, str]`` of times of day
-                (e.g. 9:00-9:30 AM) used to filter the backtesting data.
+            between_time: ``tuple[str, str]`` of times of day e.g.
+                ('9:30', '16:00') used to filter the backtesting data
+                (inclusive).
             days: Days (e.g. ``"mon"``, ``"tues"`` etc.) used to filter the
                 backtesting data.
             lookahead: Number of bars in the future of the target prediction.
@@ -1162,7 +1148,7 @@ class Strategy(
                     f"{self._end_date}."
                 )
             if start_dt is not None and end_dt is not None:
-                self._verify_date_range(start_dt, end_dt)
+                verify_date_range(start_dt, end_dt)
             self._logger.walkforward_start(start_dt, end_dt)
             df = self._fetch_data(timeframe)
             day_ids = self._to_day_ids(days)
@@ -1186,7 +1172,7 @@ class Strategy(
                 disable_parallel=disable_parallel,
             )
             train_only = all(map(lambda e: e.fn is None, self._executions))
-            test_results = self._run_walkfoward(
+            test_results = self._run_walkforward(
                 df=df,
                 indicator_data=indicator_data,
                 tf_seconds=tf_seconds,
@@ -1227,7 +1213,7 @@ class Strategy(
             self._data_source, AlpacaCrypto
         )
 
-    def _run_walkfoward(
+    def _run_walkforward(
         self,
         df: pd.DataFrame,
         indicator_data: dict[IndicatorSymbol, pd.Series],
@@ -1315,20 +1301,23 @@ class Strategy(
         days: Optional[tuple[int]],
     ) -> pd.DataFrame:
         if start_date != self._start_date or end_date != self._end_date:
-            df = _between(df, start_date, end_date)
-            df = df.reset_index(drop=True)
+            df = _between(df, start_date, end_date).reset_index(drop=True)
         if df[DataCol.DATE.value].dt.tz is not None:
             # Fixes bug on Windows.
             # https://stackoverflow.com/questions/51827582/message-exception-ignored-when-dealing-pandas-datetime-type
             df[DataCol.DATE.value] = df[DataCol.DATE.value].dt.tz_convert(None)
         is_time_range = between_time is not None or days is not None
         if is_time_range:
-            df = df.reset_index(drop=True)
-            df = df.set_index(DataCol.DATE.value)
+            df = df.reset_index(drop=True).set_index(DataCol.DATE.value)
         if days is not None:
             self._logger.info_walkforward_on_days(days)
             df = df[df.index.weekday.isin(frozenset(days))]
         if between_time is not None:
+            if len(between_time) != 2:
+                raise ValueError(
+                    "between_time must be a tuple[str, str] of start time and"
+                    f" end time, received {between_time!r}."
+                )
             self._logger.info_walkforward_between_time(between_time)
             df = df.between_time(*between_time)
         if is_time_range:
@@ -1443,11 +1432,10 @@ class Strategy(
             bootstrap_sample_size=self._config.bootstrap_sample_size,
             bootstrap_samples=self._config.bootstrap_samples,
         )
-        metrics = [
-            (name, value)
-            for name, value in dataclasses.asdict(eval_result.metrics).items()
-        ]
-        metrics_df = pd.DataFrame(metrics, columns=["name", "value"])
+        metrics_df = pd.DataFrame(
+            dataclasses.asdict(eval_result.metrics).items(),
+            columns=["name", "value"],
+        )
         self._logger.walkforward_completed()
         return TestResult(
             start_date=start_date,
