@@ -15,6 +15,12 @@ You should have received a copy of the GNU Lesser General Public License along
 with this program.  If not, see <https://www.gnu.org/licenses/>.
 """
 
+import joblib
+import numpy as np
+import os
+import pandas as pd
+import pytest
+import re
 from .fixtures import *
 from collections import deque
 from datetime import datetime
@@ -39,12 +45,6 @@ from pybroker.strategy import (
     WalkforwardMixin,
 )
 from unittest.mock import MagicMock, Mock
-import joblib
-import numpy as np
-import os
-import pandas as pd
-import pytest
-import re
 
 
 @pytest.fixture(params=[200, 202])
@@ -135,7 +135,7 @@ class TestWalkforwardMixin:
             (10, 2, 11, 0.5),
         ],
     )
-    def test_walkfoward_split_when_invalid_params_then_error(
+    def test_walkforward_split_when_invalid_params_then_error(
         self, dates, dates_length, windows, lookahead, train_size
     ):
         df = self._data_frame(dates, dates_length)
@@ -1233,7 +1233,7 @@ class TestStrategy:
                 "2020-02-20",
                 None,
                 None,
-                r"start_date (.*) must be before end_date (.*)\.",
+                r"start_date (.*) must be on or before end_date (.*)\.",
             ),
             (
                 "2020-03-01",
@@ -1268,7 +1268,7 @@ class TestStrategy:
                 "2020-09-30",
                 "2020-05-01",
                 "2020-04-01",
-                r"start_date (.*) must be before end_date (.*)\.",
+                r"start_date (.*) must be on or before end_date (.*)\.",
             ),
         ],
     )
@@ -1340,12 +1340,14 @@ class TestStrategy:
         )
         assert df.iloc[0]["date"] >= start_date
         assert df.iloc[-1]["date"] <= end_date
+        row_days = set()
         for _, row in df.iterrows():
             if between_time is not None:
                 assert row["date"].hour >= expected_hour[0]
                 assert row["date"].hour <= expected_hour[1]
-            if expected_days is not None:
-                assert row["date"].weekday() in expected_days
+            row_days.add(row["date"].weekday())
+        if expected_days is not None:
+            assert row_days == expected_days
 
     def test_filter_dates_when_empty(self, data_source_df):
         strategy = Strategy(data_source_df, START_DATE, END_DATE)
@@ -1360,6 +1362,27 @@ class TestStrategy:
         )
         assert df.empty
 
+    def test_filter_dates_when_invalid_between_time_then_error(
+        self, data_source_df
+    ):
+        strategy = Strategy(data_source_df, START_DATE, END_DATE)
+        start_date = pd.to_datetime("1/1/2021").to_pydatetime()
+        end_date = pd.to_datetime("12/1/2021").to_pydatetime()
+        with pytest.raises(
+            ValueError,
+            match=re.escape(
+                "between_time must be a tuple[str, str] of start time and end"
+                " time, received '9:00'."
+            ),
+        ):
+            strategy._filter_dates(
+                data_source_df,
+                start_date,
+                end_date,
+                days=None,
+                between_time=("9:00"),
+            )
+
     def test_add_execution_when_empty_symbols_then_error(self, data_source_df):
         strategy = Strategy(data_source_df, START_DATE, END_DATE)
         with pytest.raises(
